package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import java.util.jar.JarOutputStream

 /**
 * Jar Delegate
 * @author <a href="mailto:steve.king@upco.co.uk">Steve king</a>
 * @version 1.0.0
 * 
 * Description: A Jar Delegate for the creation of JAR/ZIP files. 
 * 
 * <p>The jar delegate read properties from either the default plugin.properties file
 * or the project's build.properties file. Based on these properties it will create
 * either a jar or a zip in the specified output folder, it is also possible to include
 * a custom manifest file if needed.</p>
 */
/* 
 * @ant-target doJar
 */ 
class JarDelegate extends AbstractPluginTargetDelegate {

	/*
	 * File name of a manifest file to include in the created jar. Leave blank to create a default manifest in the jar file.
	 * @ant-property-name jar.manifest
	 */	
	private String manifest
	
	/*
	 * Destination for output jar or zip file.
	 * @ant-property-name jar.destination.folder
	 */		
	private String destinationFolder
	
	/*
	 * Type of file to create valid types are Jar or Zip.
	 * @ant-property-name jar.type
	 */		
	private String type

	/**
	 * Constructor sets ant and buildContext instances in super class
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */	
	JarDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */	
	void doCheck(){
		assert buildContext.get( "project.properties" ) != null
		assert manifest != null
		assert destinationFolder != null
		assert type != null
		switch( type ){
			case "jar":
				assert type.equals( "jar" )
				break
			case "zip":
				assert type.equals( "zip" )
				break
			default:
				assert type.equals( "jar" ) || type.equals( "zip" )
		}
		// assert global variables exists in ant project
		// there are no global variables in this plugin
	}	
	
	/**
	 * Injector for manifest element
	 */
	void setManifest( String manifest ){
		this.manifest = manifest
	}
	
	/**
	 * Injector for destination folder for jar file.
	 * If folder string doesn't end with a slash character add one. 
	 */
	void setDestinationFolder( String destinationFolder ){
		this.destinationFolder = destinationFolder		
		if( !( destinationFolder.endsWith( '/' ) || ( destinationFolder.endsWith( '\\' ) ) ) )
				this.destinationFolder = this.destinationFolder + '/'
	}
	
	/**
	 * Injector for compressed file type (zip or jar)
	 */
	void setType( String type ){
		this.type = type
	}
	
	/**
	 * Concrete implementation of the AbstractDelegate.execute() method
	 */
	void doExecute(){		
			
		// Retrieve the indexed project properties from the global context
		Map properties = buildContext.get( "project.properties" )
		List srcProjectProperties = properties.get( "src" )
		List resourceProjectProperties = properties.get( "resources" )
		
		String jarDest = destinationFolder + ant.project.getProperty( "global.artifactid" ) + '.' + type
		if ( log.isDebugEnabled() ) {		
			log.debug( " global.arifactid : " + ant.project.getProperty( "global.artifactid" ))
			log.debug( "Creating Jar: $jarDest" )
		}
		
		def jarOptions = [destfile:jarDest]
		// check if a manifest file has been included, if not we want a default manifest building for us by ant
		if ( !manifest.equals( "" ) )
			jarOptions.put( "manifest", manifest )
		// create compressed file using ant.jar task. 
		ant.jar( jarOptions ){			
			srcProjectProperties.each{projectProperty ->				
				if ( log.isDebugEnabled() ) 
					log.debug( "Adding to Jar: $projectProperty.output" )
				ant.fileset ( dir:projectProperty.output )
			}			
			resourceProjectProperties.each{projectProperty ->				
				if ( log.isDebugEnabled() ) 
					log.debug( "Adding to Jar: $projectProperty.output" )
				ant.fileset ( dir:projectProperty.output )
			}
		}
		ant.property( name : "global.artifact.location", value : jarDest )
	}
}